import json
import click
import pathlib
import subprocess
from kube_compose import utils
from kube_compose.cli import cli

@cli.command(context_settings={"ignore_unknown_options": True})
@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
@click.option('-T', '--no-tar', type=bool, is_flag=True, default=False)
@click.argument('args', nargs=-1, type=str, metavar='[SERVICE:SRC_PATH|SRC_PATH|-] [DEST_PATH|-|SERVICE:DEST_PATH]')
def cp(*, no_tar, args, context, namespace, kubectl, **_):
  ''' Like `docker-compose cp` but for the kubernetes deployed resources
  Use `-T` as a fallback if tar is not available, cat can be used one-at-a-time instead
  '''
  args_ = []
  if no_tar:
    src, dst = args
    if ':' in src and ':' not in dst:
      dst_path = pathlib.Path(dst)
      service, _, path = src.partition(':')
      pod = json.loads(utils.check_output([
        *kubectl,
        *(['--context', context] if context else []),
        *(['-n', namespace] if namespace else []),
        'get', 'pod',
        '-l', f"app.kubernetes.io/name={service}", '-o', 'jsonpath="{.items[0][\'metadata.name\']}"'
      ]).decode())
      src_path = pathlib.PurePosixPath(path)
      ls_R = utils.check_output([
        *kubectl, 'exec',
        *(['--context', context] if context else []),
        *(['-n', namespace] if namespace else []),
        '-t',
        pod,
        '--', 'ls', '-R', path
      ])
      directory = src_path
      dirs = [directory]
      paths = []
      for line in ls_R.decode().splitlines():
        line = line.rstrip('\r\n')
        if not line:
          directory = None
          continue
        if directory is None or line.endswith(':'):
          dir, sep, end = line.rpartition(':')
          assert sep == ':' and end == ''
          directory = pathlib.PurePosixPath(dir)
          dirs.append(directory)
          continue
        else:
          paths.append(directory/line)
      files = set(paths) - set(dirs)
      for dir in dirs:
        click.echo(f"mkdir -p {str(dst_path/dir.relative_to(src_path))}")
        (dst_path/dir.relative_to(src_path)).mkdir(exist_ok=True)
      for file in files:
        click.echo(f"scp {service}:{file} {str(dst_path/(file.relative_to(src_path)))}")
        with (dst_path/(file.relative_to(src_path))).open('wb') as fw:
          subprocess.run([
            *kubectl,
            *(['--context', context] if context else []),
            *(['-n', namespace] if namespace else []),
            'exec',
            '-t',
            pod,
            '--', 'cat', str(file)
          ], stdout=fw, check=True)
    elif ':' in dst and ':' not in src:
      service, _, dst_root = dst.partition(':')
      src_path = pathlib.Path(src)
      for path in (src_path.rglob('*') if src_path.is_dir() else [src_path]):
        dst_path = pathlib.PurePosixPath(dst_root) / path.relative_to(src_path)
        if path.is_dir():
          click.echo(f"exec -- mkdir -p  {str(dst_path)}")
          subprocess.run([
            *kubectl,
            *(['--context', context] if context else []),
            *(['-n', namespace] if namespace else []),
            'exec',
            '-t',
            pod,
            '--', 'mkdir', '-p', str(dst_path)
          ], check=True)
        elif path.is_file():
          click.echo(f"exec -- 'cat > {str(dst_path)}' < {str(path)}")
          with path.open('rb') as fr:
            subprocess.run([
              *kubectl,
              *(['--context', context] if context else []),
              *(['-n', namespace] if namespace else []),
              'exec',
              '-i',
              pod,
              '--', '/bin/sh', '-c', f"cat > {str(dst_path)}"
            ], stdin=fr, check=True)
        else:
          click.echo(f"Ignoring special file {str(path)}")
    else:
      raise click.UsageError('Only supports host => container or container => host')
  else:
    for arg in args:
      if ':' in arg:
        service, _, path = arg.partition(':')
        # find the pod for the specified docker-compose service
        pod = json.loads(utils.check_output([
          *kubectl,
          *(['--context', context] if context else []),
          *(['-n', namespace] if namespace else []),
          'get', 'pod',
          '-l', f"app.kubernetes.io/name={service}", '-o', 'jsonpath="{.items[0][\'metadata.name\']}"'
        ]).decode())
        args_.append(
          (f"{namespace}/" if namespace else '')
          + f"{pod}:{path}"
        )
      else:
        args_.append(arg)
    utils.run([
      *kubectl,
      *(['--context', context] if context else []),
      'cp',
      *args_,
    ])



import os
import yaml
import click
from kube_compose import utils
from kube_compose.cli.configmap import configmap

def create_configmap_spec(*, configmap, docker_compose_path, docker_compose_config, **_):
  configmap_specs = []
  for config, config_config in ([(config, docker_compose_config['configs'][configmap],)] if configmap is not None else docker_compose_config.get('configs', {}).items()):
    config_ext_config = config_config.get('x-kubernetes', {})
    configmap_spec = {
      'apiVersion': 'v1',
      'kind': 'ConfigMap',
      'metadata': {
        'name': config,
        'labels': dict({'app.kubernetes.io/managed-by': 'kube-compose'}, **config_ext_config.get('labels', {})),
        'annotations': config_ext_config.get('annotations', {}),
      },
      'data': {}
    }
    if 'content' in config_config:
      configmap_spec['data']['content'] = config_config['content']
    elif 'file' in config_config:
      configmap_spec['data']['content'] = (docker_compose_path.parent / config_config['file']).read_text()
    elif 'environment' in config_config:
      configmap_spec['data']['content'] = os.environ[config_config['environment']]
    elif 'external' in config_config:
      pass
    else:
      raise NotImplementedError('config could not be located')
    configmap_specs.append(configmap_spec)
  return configmap_specs

@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
def create(*, configmap, docker_compose_path, docker_compose_config, context, namespace, kubectl, **_):
  configmap_specs = create_configmap_spec(
    configmap=configmap,
    docker_compose_path=docker_compose_path,
    docker_compose_config=docker_compose_config,
    kubectl=kubectl,
  )
  if not configmap_specs: return
  utils.run([
    *kubectl,
    *(['--context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'apply',
    '-f', '-',
  ], input=yaml.dump_all(configmap_specs).encode())

@configmap.command('create')
@click.argument('configmap', type=str, required=False)
def _(**kwargs):
  ''' Ensure a kubernetes configmap is created
  '''
  create(**kwargs)



from kube_compose.cli import cli
from kube_compose import utils

@cli.command()
@utils.require_binaries(helm='helm')
@utils.require_kube_compose_release
def ps(*, name, context, namespace, helm, **_):
  ''' Like `docker-compose ps` but show status of the kubernetes deployed resource
  '''
  utils.run([
    *helm,
    *(['--kube-context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'status',
    name,
    '--show-resources',
  ])



import re
import click
from ruamel.yaml import YAML
from kube_compose.cli import cli
from kube_compose import utils

@cli.command()
@click.argument('service', type=str, default=None)
@click.argument('newversion', type=str, default=None, metavar='[<newversion> | major | minor | patch]', required=False)
def version(service, newversion):
  ''' Get or set the version of a service in docker-compose.yaml
  '''
  yaml = YAML()
  docker_compose_path = utils.locate_docker_compose_path()
  with docker_compose_path.open('r') as fr:
    docker_compose_config = yaml.load(fr)
  #
  service = docker_compose_config['services'][service]
  container, sep, version = service['image'].partition(':')
  if not sep:
    sep = ':'
    version = 'latest'
  if not newversion:
    click.echo(version)
    return
  elif newversion in ('major', 'minor', 'patch'):
    semver = re.match(r'^(.*?)(\d+)\.(\d+)(\.(\d+))?(.*?)$', version)
    if not semver: raise RuntimeError(f"Failed to parse {version}")
    prefix, major, minor, _, patch, postfix = semver.groups()
    if newversion == 'major':
      major = str(int(major)+1)
      minor = '0'
      patch = '0'
    elif newversion == 'minor':
      minor = str(int(minor)+1)
      patch = '0'
    elif newversion == 'patch':
      patch = str(int(patch or -1)+1)
    newversion = ''.join([prefix, '.'.join(filter(None, [major, minor, patch])), postfix])
    service['image'] = f"{container}{sep}{newversion}"
  elif newversion:
    service['image'] = f"{container}{sep}{newversion}"
  with docker_compose_path.open('w') as fw:
    yaml.dump(docker_compose_config, fw)



import sys
import yaml
import click
from kube_compose import utils
from kube_compose.cli.volume import volume
from kube_compose.cli.volume.create import create_volume_spec

@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
def template(*, volume, docker_compose_config, context, namespace, kubectl, **_):
  volume_specs = create_volume_spec(
    volume=volume,
    docker_compose_config=docker_compose_config,
    kubectl=kubectl,
  )
  if not volume_specs: return []
  # use kubectl to fill-in defaults/validate
  templ = yaml.safe_load(utils.check_output([
    *kubectl,
    *(['--context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'apply',
    '--dry-run=client',
    '-oyaml',
    '-f', '-',
  ], input=yaml.dump_all(create_volume_spec(
    volume=volume,
    docker_compose_config=docker_compose_config,
    kubectl=kubectl,
  )).encode()))
  # write items in flat yaml format
  if templ['kind'] == 'List':
    return templ['items']
  else:
    return [templ]

@volume.command('template')
@click.argument('volume', type=str, required=False)
def _(**kwargs):
  ''' Ensure a kubernetes persistent volume is templated
  '''
  yaml.dump_all(template(**kwargs), sys.stdout)



import click
import itertools
from kube_compose.cli import cli
from kube_compose import utils

@cli.command()
@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
@click.argument('args', nargs=-1, type=str)
def scale(args, *, context, namespace, kubectl, deployments, **_):
  ''' Like `docker-compose scale` but effects the kubernetes deployed resources
  Arguments are expected to be like: `service-name=replicas`
  '''
  deploy_replicas = []
  for arg in args:
    service_name, _, replicas = arg.partition('=')
    deploy_replicas.append((int(replicas), deployments[service_name]))
  #
  for replicas, deploy in itertools.groupby(sorted(deploy_replicas), lambda key: key[0]):
    utils.run([
      *kubectl,
      *(['--context', context] if context else []),
      *(['-n', namespace] if namespace else []),
      'scale',
      f"--replicas={replicas}",
      *[d for _, d in deploy],
    ])



import yaml
from kube_compose.cli import cli
from kube_compose import utils

@cli.command()
@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
def drift(*, kubectl, context, namespace, **_):
  ''' Compare the expected next deployment with the current deployment in kubernetes
  '''
  from kube_compose.cli.template import template
  utils.run([
    *kubectl,
    *(['--context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'diff', '-f', '-'
  ], input=yaml.dump_all(template()).encode())



import click
from kube_compose import utils
from kube_compose.cli.volume import volume

@volume.command('inspect')
@click.argument('volume', type=str, required=True)
def _(**kwargs):
  ''' Inspect a kubernetes persistent volume
  '''
  inspect(**kwargs)

@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
def inspect(*, volume, context, namespace, kubectl, **_):
  utils.run([
    *kubectl,
    *(['--context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'get',
    f"pvc/{volume}",
    '-o', 'yaml',
  ])



import click
import yaml
from kube_compose import utils
from kube_compose.cli.volume import volume

def create_volume_spec(*, volume, docker_compose_config, **_):
  volume_specs = []
  for volume, volume_config in ([(volume, docker_compose_config['volumes'][volume],)] if volume is not None else docker_compose_config.get('volumes', {}).items()):
    volume_ext_config = volume_config.get('x-kubernetes', {})
    volume_spec = {
      'apiVersion': 'v1',
      'kind': 'PersistentVolumeClaim',
      'metadata': {
        'name': volume,
        'labels': dict({'app.kubernetes.io/managed-by': 'kube-compose'}, **volume_ext_config.get('labels', {})),
        'annotations': volume_ext_config.get('annotations', {}),
      },
      'spec': {
        'accessModes': [volume_ext_config.get('mode', 'ReadWriteOnce')],
        'resources': {
          'requests': {
            'storage': volume_ext_config.get('size', '1Gi'),
          },
        },
        'storageClassName': volume_ext_config.get('class', ''),
        'volumeMode': 'Filesystem',
      },
    }
    if 'name' in volume_ext_config:
      volume_spec['spec']['volumeName'] = volume_ext_config['name']
    volume_specs.append(volume_spec)
  return volume_specs

@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
def create(*, volume, docker_compose_config, context, namespace, kubectl, **_):
  volume_specs = create_volume_spec(
    volume=volume,
    docker_compose_config=docker_compose_config,
    kubectl=kubectl,
  )
  if not volume_specs: return
  utils.run([
    *kubectl,
    *(['--context', context] if context else []),
    *(['-n', namespace] if namespace else []),
     'apply',
    '-f', '-',
  ], input=yaml.dump_all(volume_specs).encode())

@volume.command('create')
@click.argument('volume', type=str, required=False)
def _(**kwargs):
  ''' Ensure a kubernetes persistent volume is created
  '''
  create(**kwargs)



import click
from kube_compose import utils
from kube_compose.cli.configmap import configmap

@configmap.command('rm')
@click.argument('configmap', type=str, required=False)
def _(**kwargs):
  ''' Remove a kubernetes configmap
  '''
  rm(**kwargs)

@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
def rm(*, configmap, docker_compose_config, context, namespace, kubectl, **_):
  for configmap in ([configmap] if configmap is not None else docker_compose_config.get('configs', {}).keys()):
    utils.run([
      *kubectl,
      *(['--context', context] if context else []),
      *(['-n', namespace] if namespace else []),
      'delete',
      f"configmap/{configmap}",
    ])



import click
from kube_compose.cli import cli
from kube_compose import utils

@cli.command()
@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
@click.argument('service', type=str, required=False)
def restart(service, *, context, namespace, kubectl, deployments, **_):
  ''' Like `docker-compose restart` but effects the kubernetes deployed resources
  '''
  deploy = [deployments[service]] if service is not None else list(deployments.values())
  utils.run([
    *kubectl,
    *(['--context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'rollout', 'restart',
    *deploy,
  ])
  utils.run([
    *kubectl,
    *(['--context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'rollout', 'status',
    *deploy,
  ])



import json
import uuid
import click
from kube_compose import utils
from kube_compose.cli import cli

@cli.command(context_settings={"ignore_unknown_options": True})
@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
@click.option('-i', '--stdin', type=bool, is_flag=True)
@click.option('-t', '--tty', type=bool, is_flag=True)
@click.argument('service', type=str)
@click.argument('args', nargs=-1, type=str)
def run(*, service, args, stdin, tty, context, namespace, docker_compose_config, kubectl, **_):
  ''' Like `docker-compose run` but for the kubernetes deployed resources
  '''
  service_config = docker_compose_config['services'][service]
  name = f"{service}-{uuid.uuid4()}"
  overrides = {
    'spec': {
      'containers': [{
        'name': name,
        'image': service_config['image'],
      }],
    },
  }
  #
  if service_config.get('environment'):
    overrides['spec']['containers'][0]['env'] = [
      {
        'name': key,
        'value': value,
      }
      for key, value in service_config.get('environment', {}).items()
    ]
  #
  if service_config.get('volumes'):
    overrides['spec']['containers'][0]['volumeMounts'] = []
    overrides['spec']['volumes'] = []
    for volume in service_config['volumes']:
      from kube_compose.cli.volume.create import create
      create(volume=volume['source'])
      overrides['spec']['containers'][0]['volumeMounts'].append(
        {
          'mountPath': volume['target'],
          'name': volume['source'],
        }
      )
      overrides['spec']['volumes'].append(
        {
          'name': volume['source'],
          'persistentVolumeClaim': {
            'claimName': volume['source'],
          },
        }
      )
  #
  utils.run([
    *kubectl,
      *(['--context', context] if context else []),
      *(['-n', namespace] if namespace else []),
      'run',
      name,
      '--attach',
      '--rm',
      *(('-i',) if stdin else tuple()),
      *(('-t',) if tty else tuple()),
      f"--image={service_config['image']}",
      '--overrides', json.dumps(overrides),
      *(('--command', '--', *args) if args else tuple()),
  ])



import click
from kube_compose import utils
from kube_compose.cli.volume import volume

@volume.command('rm')
@click.argument('volume', type=str, required=False)
def _(**kwargs):
  ''' Remove a kubernetes persistent volume
  '''
  rm(**kwargs)

@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
def rm(*, volume, docker_compose_config, context, namespace, kubectl, **_):
  for volume in ([volume] if volume is not None else docker_compose_config.get('volumes', {}).keys()):
    utils.run([
      *kubectl,
      *(['--context', context] if context else []),
      *(['-n', namespace] if namespace else []),
      'delete',
      f"pvc/{volume}",
    ])



import click
import subprocess
from kube_compose.cli import cli
from kube_compose import utils

@cli.command()
@utils.require_binaries(helm='helm', kubectl='kubectl')
@utils.require_kube_compose_release
@click.option('--create-namespace', type=bool, is_flag=True, help='Create the release namespace if not present')
def up(*, helm, kubectl, name, context, namespace, docker_compose_config_raw, deployments, create_namespace, **_):
  ''' Like `docker-compose up` but runs in the kubernetes cluster
  '''
  from kube_compose.cli.volume.create import create as volume_create
  from kube_compose.cli.configmap.create import create as configmap_create
  volume_create(volume=None)
  configmap_create(configmap=None)
  if subprocess.call([
    *helm,
    *(['--kube-context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'status',
    name
  ], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL) == 0:
    utils.run([
      *helm,
      *(['--kube-context', context] if context else []),
      *(['-n', namespace] if namespace else []),
      'upgrade',
      name,
      utils.helm_chart,
      '-f', '-'
    ], input=docker_compose_config_raw)
  else:
    utils.run([
      *helm,
      *(['--kube-context', context] if context else []),
      *(['-n', namespace] if namespace else []),
      'install',
      name,
      utils.helm_chart,
      *(['--create-namespace'] if create_namespace else []),
      '-f', '-',
    ], input=docker_compose_config_raw)
  #
  utils.run([
    *kubectl,
    *(['--context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'rollout', 'status',
    *deployments.values(),
  ])



import sys
import yaml
from kube_compose.cli import cli
from kube_compose import utils

@cli.command('template')
def _(**_):
  ''' Alias for `helm template [release]`
  '''
  from kube_compose.cli.volume.template import template as volume_create_template
  from kube_compose.cli.configmap.template import template as configmap_create_template
  yaml.dump_all([
    *volume_create_template(volume=None),
    *configmap_create_template(configmap=None),
    *template(),
  ], sys.stdout)

@utils.require_binaries(helm='helm')
@utils.require_kube_compose_release
def template(*, helm, name, context, namespace, docker_compose_config_raw, **_):
  output = list(yaml.safe_load_all(utils.check_output([
    *helm,
    *(['--kube-context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'template',
    name,
    utils.helm_chart,
    '-f', '-', '--validate=false',
  ], input=docker_compose_config_raw)))
  return output



import click
import pathlib
from ruamel.yaml import YAML
from kube_compose.cli import cli
from kube_compose import utils

@cli.command()
def init():
  ''' Initialize a kube-compose configuration
  '''
  yaml = YAML()
  try:
    docker_compose_path = utils.locate_docker_compose_path()
  except FileNotFoundError:
    docker_compose_path = pathlib.Path('docker-compose.yaml')
  try:
    with docker_compose_path.open('r') as fr:
      docker_compose_config = yaml.load(fr)
  except FileNotFoundError:
    docker_compose_config = {
      'services': {},
    }
  #
  if 'x-kubernetes' not in docker_compose_config:
    docker_compose_config['x-kubernetes'] = {}
  release_config = docker_compose_config['x-kubernetes']
  if 'name' not in release_config:
    release_config['name'] = click.prompt('Release Name', type=str, default=docker_compose_path.absolute().parent.name)
  if 'namespace' not in release_config:
    release_config['namespace'] = click.prompt('Release Namespace', type=str, default='default')
  #
  with docker_compose_path.open('w') as fw:
    yaml.dump(docker_compose_config, fw)



from kube_compose import utils
from kube_compose.cli.configmap import configmap

@configmap.command('ls')
def _(**kwargs):
  ''' List kubernetes configmaps
  '''
  ls(**kwargs)

@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
def ls(*, context, namespace, kubectl, **_):
  utils.run([
    *kubectl,
    *(['--context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'get',
    'configmap',
  ])



import click
from kube_compose.cli import cli
from kube_compose import utils

@cli.command()
@utils.require_binaries(helm='helm')
@utils.require_kube_compose_release
@click.argument('revision', type=int, required=False)
def rollback(revision, *, helm, name, context, namespace, **_):
  ''' Alias for `helm rollback [revision]`
  '''
  utils.run([
    *helm,
    *(['--kube-context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'rollback',
     name,
     *([revision] if revision else []),
  ])



import click
import json
from kube_compose.cli import cli
from kube_compose import utils

@cli.command()
@click.option('-c', '--context', help='The kubernetes context to restrict ls to')
@click.option('-n', '--namespace', help='The namespace to restrict ls to')
@click.option('-A', '--all-namespaces', help='All namespaces?')
@utils.require_binaries(helm='helm')
def ls(*, context, namespace, all_namespaces, helm, **_):
  ''' Like `docker-compose ls` but shows the docker-compose releases
  '''
  utils.echo_table_via_pager(
    [('NAME', 'NAMESPACE', 'REVISION', 'UPDATED', 'STATUS')] + [
    (release['name'], release['namespace'], release['revision'], release['updated'], release['status'])
    for release in json.loads(
      utils.check_output([
        *helm,
        *(['--kube-context', context] if context else []),
        *(['-A'] if all_namespaces else (['-n', namespace] if namespace else [])),
        'list',
        '-o', 'json',
      ])
    )
    if release['chart'].startswith('kube-compose-')
  ])



import click
import itertools
from kube_compose.cli import cli
from kube_compose import utils

@cli.command()
@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
@click.argument('service', type=str, required=False)
def start(service, *, context, namespace, kubectl, docker_compose_config, deployments, **_):
  ''' Like `docker-compose start` but effects the kubernetes deployed resources
  '''
  if service is not None:
    replicas = docker_compose_config.get('services', {}).get(service, {}).get('deploy', {}).get('replicas', 1) or 1
    utils.run([
      *kubectl,
      *(['--context', context] if context else []),
      *(['-n', namespace] if namespace else []),
      'scale',
      f"--replicas={replicas}",
      deployments[service],
    ])
  else:
    deploy_replicas = []
    for svc, deploy in deployments.items():
      replicas = docker_compose_config['services'][svc].get('deploy', {}).get('replicas', 1)
      deploy_replicas.append((replicas, deploy))
    for replicas, deploy in itertools.groupby(sorted(deploy_replicas), lambda key: key[0]):
      utils.run([
        *kubectl,
        *(['--context', context] if context else []),
        *(['-n', namespace] if namespace else []),
        'scale',
        f"--replicas={replicas}",
        *[d for _, d in deploy],
      ])



import click
from kube_compose.cli import cli
from kube_compose import utils

@cli.command()
@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
@click.argument('service', type=str, required=True)
def kill(service, *, context, namespace, kubectl, docker_compose_config, **_):
  ''' Like `docker-compose kill` but effects the kubernetes deployed resources
  '''
  utils.run([
    *kubectl,
    *(['--context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'delete', 'pod',
    '-l', f"app.kubernetes.io/name={service}",
  ])



from kube_compose.cli.get import get
from kube_compose import utils

@get.command('all')
@utils.require_binaries(helm='helm')
@utils.require_kube_compose_release
def all(*, name, context, namespace, helm, **_):
  ''' download all information for a named release
  '''
  utils.run([
    *helm,
    *(['--kube-context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'get', 'all',
    name,
  ])



import click
from kube_compose.cli import cli
from kube_compose import utils

@cli.command()
@click.option('-v', '--volumes', type=bool, is_flag=True, help='Remove named volumes/configmaps declared in the "volumes"/"configs" sections')
@utils.require_binaries(helm='helm')
@utils.require_kube_compose_release
def down(*, helm, name, context, namespace, volumes, **_):
  ''' Like `docker-compose down` but stops the kubernetes deployed resources
  '''
  utils.run([
    *helm,
    *(['--kube-context', context] if context else []),
    *(['-n', namespace] if namespace else []),
    'uninstall',
    name,
  ])
  if volumes:
    from kube_compose.cli.volume.rm import rm as volume_rm
    from kube_compose.cli.configmap.rm import rm as configmap_rm
    volume_rm(volume=None)
    configmap_rm(configmap=None)



import click
from kube_compose.cli import cli
from kube_compose import utils

@cli.command()
@utils.require_binaries(kubectl='kubectl')
@utils.require_kube_compose_release
@click.argument('service', type=str, required=True)
def events(service, *, context, namespace, kubectl, deployments, **_):
  ''' Like `docker-compose events` but show events of the kubernetes deployed resource
  '''
  if service:
    utils.run([
      *kubectl,
      *(['--context', context] if context else []),
      *(['-n', namespace] if namespace else []),
      'events', 
      '--for', deployments[service],
      '--watch',
    ])
